"""Top-level package for SPADE PubSub."""

__author__ = """Javi Palanca"""
__email__ = "jpalanca@dsic.upv.es"
__version__ = "0.3.0"


from .pubsub import PubSubMixin

__all__ = ["PubSubMixin", "__version__"]
